#include "helloworld.hpp"

using namespace std;


helloworld::helloworld(DDSDomainParticipant *participant) : participant_(participant)
{

    string sysconf_dir(SYSCONFDIR);
    qos_file_ += sysconf_dir + "qos/helloworld_subscriber_qos.xml";
    cout << qos_file_ << endl;

	status_ = 0;
	count_ = 0;
	sample_count_ = 0;

	/* To customize the subscriber QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	subscriber_ = participant_->create_subscriber(
			DDS_SUBSCRIBER_QOS_DEFAULT, NULL /* listener */, DDS_STATUS_MASK_NONE);
	if (subscriber_ == NULL) {
		printf("create_subscriber error\n");
	}

	/* Register the type before creating the topic */
	type_name_ = helloworld_messageTypeSupport::get_type_name();
	retcode_ = helloworld_messageTypeSupport::register_type(
			participant_, type_name_);
	if (retcode_ != DDS_RETCODE_OK) {
		printf("register_type error %d\n", retcode_);
	}

	/* To customize the topic QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	topic_ = participant_->create_topic(
			"HelloWorld_message",
			type_name_, DDS_TOPIC_QOS_DEFAULT, NULL /* listener */,
			DDS_STATUS_MASK_NONE);
	if (topic_ == NULL) {
		printf("create_topic error\n");
	}

}


void helloworld::receive_information(void) 
{
	/* Create a data reader listener */
	reader_listener_ = new helloworld_messageListener();

	/* To customize the data reader QoS, use
       the configuration file USER_QOS_PROFILES.xml */
	reader_ = subscriber_->create_datareader(
			topic_, DDS_DATAREADER_QOS_DEFAULT, reader_listener_,
			DDS_STATUS_MASK_ALL);

	if (reader_ == NULL) {
		printf("create_datareader error\n");
		delete reader_listener_;
	}


}

void helloworld_messageListener::on_data_available(DDSDataReader* reader)
{
	helloworld_messageDataReader *helloworld_message_reader = NULL;
	helloworld_messageSeq data_seq;
	DDS_SampleInfoSeq info_seq;
	DDS_ReturnCode_t retcode;
	int i;

	helloworld_message_reader = helloworld_messageDataReader::narrow(reader);
	if (helloworld_message_reader == NULL) {
		printf("DataReader narrow error\n");
		return;
	}

	retcode = helloworld_message_reader->take(
			data_seq, info_seq, DDS_LENGTH_UNLIMITED,
			DDS_ANY_SAMPLE_STATE, DDS_ANY_VIEW_STATE, DDS_ANY_INSTANCE_STATE);

	if (retcode == DDS_RETCODE_NO_DATA) {
		return;
	} else if (retcode != DDS_RETCODE_OK) {
		printf("take error %d\n", retcode);
		return;
	}



	for (i = 0; i < data_seq.length(); ++i) {
		if (info_seq[i].valid_data) {
        	openlog(basename("Cave Canem Hello World"), LOG_PID, LOG_LOCAL3);
        	syslog(LOG_WARNING,"%s",data_seq[i].message);
        	closelog();
		}
	}

	retcode = helloworld_message_reader->return_loan(data_seq, info_seq);
	if (retcode != DDS_RETCODE_OK) {
		printf("return loan error %d\n", retcode);
	}
}

